@@ -1,11 +1,22 @@
 import sys
+from contextlib import nullcontext
 from math import ceil
 from types import LambdaType
-from typing import Callable, TYPE_CHECKING, Generator, Optional, overload, TypeVar
+from typing import (
+    Callable,
+    TYPE_CHECKING,
+    Generator,
+    Optional,
+    overload,
+    TypeVar,
+    Union,
+)
 
 if TYPE_CHECKING:
-    from ...types import T
+    from ...typing import T
     from ... import Document, DocumentArray
+    from multiprocessing.pool import ThreadPool, Pool
+
 
 T_DA = TypeVar('T_DA')
 
@@ -20,6 +31,7 @@ class ParallelMixin:
         backend: str = 'thread',
         num_worker: Optional[int] = None,
         show_progress: bool = False,
+        pool: Optional[Union['Pool', 'ThreadPool']] = None,
     ) -> 'T':
         """Apply each element in itself with ``func``, return itself after modified.
 
@@ -35,6 +47,7 @@ class ParallelMixin:
                 and the original object do **not** share the same memory.
 
         :param num_worker: the number of parallel workers. If not given, then the number of CPUs in the system will be used.
+        :param pool: use an existing/external pool. If given, `backend` is ignored and you will be responsible for closing the pool.
         :param show_progress: show a progress bar
 
         """
@@ -47,11 +60,8 @@ class ParallelMixin:
         # noqa: DAR201
         :return: a new :class:`DocumentArray`
         """
-        from ... import DocumentArray
-
-        new_da = DocumentArray(self.map(*args, **kwargs))
-        self.clear()
-        self.extend(new_da)
+        for doc in self.map(*args, **kwargs):
+            self[doc.id] = doc
         return self
 
     def map(
@@ -60,6 +70,7 @@ class ParallelMixin:
         backend: str = 'thread',
         num_worker: Optional[int] = None,
         show_progress: bool = False,
+        pool: Optional[Union['Pool', 'ThreadPool']] = None,
     ) -> Generator['T', None, None]:
         """Return an iterator that applies function to every **element** of iterable in parallel, yielding the results.
 
@@ -81,21 +92,26 @@ class ParallelMixin:
 
         :param num_worker: the number of parallel workers. If not given, then the number of CPUs in the system will be used.
         :param show_progress: show a progress bar
+        :param pool: use an existing/external pool. If given, `backend` is ignored and you will be responsible for closing the pool.
 
         :yield: anything return from ``func``
         """
-        if _is_lambda_or_local_function(func) and backend == 'process':
+        if _is_lambda_or_partial_or_local_function(func) and backend == 'process':
             func = _globalize_lambda_function(func)
 
-        if show_progress:
-            from rich.progress import track as _track
+        from rich.progress import track
 
-            track = lambda x: _track(x, total=len(self))
+        if pool:
+            p = pool
+            ctx_p = nullcontext()
         else:
-            track = lambda x: x
+            p = _get_pool(backend, num_worker)
+            ctx_p = p
 
-        with _get_pool(backend, num_worker) as p:
-            for x in track(p.imap(func, self)):
+        with ctx_p:
+            for x in track(
+                p.imap(func, self), total=len(self), disable=not show_progress
+            ):
                 yield x
 
     @overload
@@ -107,6 +123,7 @@ class ParallelMixin:
         num_worker: Optional[int] = None,
         shuffle: bool = False,
         show_progress: bool = False,
+        pool: Optional[Union['Pool', 'ThreadPool']] = None,
     ) -> 'T':
         """Apply each element in itself with ``func``, return itself after modified.
 
@@ -125,6 +142,7 @@ class ParallelMixin:
         :param batch_size: Size of each generated batch (except the last one, which might be smaller, default: 32)
         :param shuffle: If set, shuffle the Documents before dividing into minibatches.
         :param show_progress: show a progress bar
+        :param pool: use an existing/external pool. If given, `backend` is ignored and you will be responsible for closing the pool.
 
         """
         ...
@@ -136,13 +154,8 @@ class ParallelMixin:
         # noqa: DAR201
         :return: a new :class:`DocumentArray`
         """
-        from ... import DocumentArray
-
-        new_da = DocumentArray()
         for _b in self.map_batch(*args, **kwargs):
-            new_da.extend(_b)
-        self.clear()
-        self.extend(new_da)
+            self[[doc.id for doc in _b]] = _b
         return self
 
     def map_batch(
@@ -153,6 +166,7 @@ class ParallelMixin:
         num_worker: Optional[int] = None,
         shuffle: bool = False,
         show_progress: bool = False,
+        pool: Optional[Union['Pool', 'ThreadPool']] = None,
     ) -> Generator['T', None, None]:
         """Return an iterator that applies function to every **minibatch** of iterable in parallel, yielding the results.
         Each element in the returned iterator is :class:`DocumentArray`.
@@ -177,23 +191,28 @@ class ParallelMixin:
 
         :param num_worker: the number of parallel workers. If not given, then the number of CPUs in the system will be used.
         :param show_progress: show a progress bar
+        :param pool: use an existing/external pool. If given, `backend` is ignored and you will be responsible for closing the pool.
 
         :yield: anything return from ``func``
         """
 
-        if _is_lambda_or_local_function(func) and backend == 'process':
+        if _is_lambda_or_partial_or_local_function(func) and backend == 'process':
             func = _globalize_lambda_function(func)
 
-        if show_progress:
-            from rich.progress import track as _track
+        from rich.progress import track
 
-            track = lambda x: _track(x, total=ceil(len(self) / batch_size))
+        if pool:
+            p = pool
+            ctx_p = nullcontext()
         else:
-            track = lambda x: x
+            p = _get_pool(backend, num_worker)
+            ctx_p = p
 
-        with _get_pool(backend, num_worker) as p:
+        with ctx_p:
             for x in track(
-                p.imap(func, self.batch(batch_size=batch_size, shuffle=shuffle))
+                p.imap(func, self.batch(batch_size=batch_size, shuffle=shuffle)),
+                total=ceil(len(self) / batch_size),
+                disable=not show_progress,
             ):
                 yield x
 
@@ -213,9 +232,11 @@ def _get_pool(backend, num_worker):
         )
 
 
-def _is_lambda_or_local_function(func):
-    return (isinstance(func, LambdaType) and func.__name__ == '<lambda>') or (
-        '<locals>' in func.__qualname__
+def _is_lambda_or_partial_or_local_function(func):
+    return (
+        (isinstance(func, LambdaType) and func.__name__ == '<lambda>')
+        or not hasattr(func, '__qualname__')
+        or ('<locals>' in func.__qualname__)
     )
 
 
