@@ -5,12 +5,7 @@ import random
 import sys
 import uuid
 import warnings
-from typing import Any, Dict, Optional, Sequence, Tuple
-
-ALLOWED_PROTOCOLS = {'pickle', 'protobuf', 'protobuf-array', 'pickle-array'}
-ALLOWED_COMPRESSIONS = {'lz4', 'bz2', 'lzma', 'zlib', 'gzip'}
-
-__windows__ = sys.platform == 'win32'
+from typing import Any, Dict, Optional, Sequence, Tuple, Union
 
 __resources_path__ = os.path.join(
     os.path.dirname(
@@ -396,6 +391,9 @@ def protocol_and_compress_from_file_path(
     (None, gzip)
     """
 
+    ALLOWED_PROTOCOLS = {'pickle', 'protobuf', 'protobuf-array', 'pickle-array'}
+    ALLOWED_COMPRESSIONS = {'lz4', 'bz2', 'lzma', 'zlib', 'gzip'}
+
     protocol = default_protocol
     compress = default_compress
 
@@ -434,3 +432,23 @@ def add_protocol_and_compress_to_file_path(
         file_path_extended += '.' + compress
 
     return file_path_extended
+
+
+def filter_dict(d: Dict) -> Dict:
+    """Removes `None` values from dict `d`.
+
+    :param d: input dict
+    :return: filtered dict
+    """
+    return dict(filter(lambda item: item[1] is not None, d.items()))
+
+
+def _safe_cast_int(value: Union[str, int, float]) -> int:
+    """Safely cast string and float to an integer
+    It mainly avoids silently rounding down the float value
+    :param value: value to be cast
+    :return: cast integer
+    """
+    if isinstance(value, float) and not value.is_integer():
+        raise ValueError(f"Can't safely cast {value} to an int")
+    return int(value)
